Package: lavaSearch2
Type: Package
Title: Tools for Model Specification in the Latent Variable Framework
Version: 2.0.3
Date: 2024-02-22
Authors@R: c(
    person("Brice", "Ozenne", role = c("aut", "cre"), email = "brice.mh.ozenne@gmail.com", comment = c(ORCID = "0000-0001-9694-2956"))
	)
URL: https://github.com/bozenne/lavaSearch2
BugReports: https://github.com/bozenne/lavaSearch2/issues
Description: Tools for model specification in the latent variable framework
    (add-on to the 'lava' package). The package contains three main functionalities:
    Wald tests/F-tests with improved control of the type 1 error in small samples,
    adjustment for multiple comparisons when searching for local dependencies,
    and adjustment for multiple comparisons when doing inference for multiple latent variable models. 
License: GPL-3
Encoding: UTF-8
Depends: R (>= 2.10), ggplot2, lava (>= 1.6.4)
Imports: abind, doParallel, MASS, Matrix, methods, multcomp, mvtnorm,
        nlme, parallel, Rcpp, reshape2, sandwich, stats, utils
Suggests: clubSandwich, data.table, foreach, emmeans, lme4, lmerTest,
        numDeriv, pbapply, pbkrtest, R.rsp, riskRegression, survival,
        testthat
LinkingTo: Rcpp, RcppArmadillo
VignetteBuilder: R.rsp
NeedsCompilation: yes
RoxygenNote: 7.2.3
Packaged: 2024-02-22 11:18:47 UTC; hpl802
Author: Brice Ozenne [aut, cre] (<https://orcid.org/0000-0001-9694-2956>)
Maintainer: Brice Ozenne <brice.mh.ozenne@gmail.com>
Repository: CRAN
Date/Publication: 2024-02-23 09:10:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-28 05:54:20 UTC; unix
Archs: lavaSearch2.so.dSYM
