# Generated by roxygen2: do not edit by hand

S3method("sCorrect<-",default)
S3method(addLink,lvm)
S3method(addLink,lvm.reduced)
S3method(autoplot,calibrateType1)
S3method(autoplot,intDensTri)
S3method(autoplot,modelsearch2)
S3method(calibrateType1,lvm)
S3method(calibrateType1,lvmfit)
S3method(checkData,lvm)
S3method(clean,lvm)
S3method(coef,lvmfit2)
S3method(coef2,lvmfit)
S3method(coef2,lvmfit2)
S3method(coefCov,lvm)
S3method(coefCov,lvmfit)
S3method(coefCov,multigroup)
S3method(coefExtra,lvm)
S3method(coefExtra,lvmfit)
S3method(coefExtra,multigroup)
S3method(coefIndexModel,lvm)
S3method(coefIndexModel,lvmfit)
S3method(coefIndexModel,multigroup)
S3method(coefIndexModel,multigroupfit)
S3method(coefIntercept,lvm)
S3method(coefIntercept,lvmfit)
S3method(coefIntercept,multigroup)
S3method(coefRef,lvmfit)
S3method(coefReg,lvm)
S3method(coefReg,lvmfit)
S3method(coefReg,multigroup)
S3method(coefType,lvm)
S3method(coefType,lvmfit)
S3method(coefType,multigroup)
S3method(coefVar,lvm)
S3method(coefVar,lvmfit)
S3method(coefVar,multigroup)
S3method(compare,lvmfit2)
S3method(compare2,lvmfit)
S3method(compare2,lvmfit2)
S3method(confint,lvmfit2)
S3method(confint2,lvmfit)
S3method(confint2,lvmfit2)
S3method(createContrast,character)
S3method(createContrast,list)
S3method(createContrast,lvmfit)
S3method(createContrast,lvmfit2)
S3method(createContrast,mmm)
S3method(effects,lvmfit2)
S3method(effects2,lvmfit)
S3method(effects2,lvmfit2)
S3method(estimate2,list)
S3method(estimate2,lvm)
S3method(estimate2,lvmfit)
S3method(estimate2,mmm)
S3method(extractData,lvmfit)
S3method(findNewLink,lvm)
S3method(getNewLink,modelsearch2)
S3method(getNewModel,modelsearch2)
S3method(getStep,modelsearch2)
S3method(getVarCov2,lvmfit)
S3method(getVarCov2,lvmfit2)
S3method(glht,lvmfit2)
S3method(glht2,lvmfit)
S3method(glht2,lvmfit2)
S3method(glht2,mmm)
S3method(hessian2,lvmfit)
S3method(hessian2,lvmfit2)
S3method(iid,lvmfit2)
S3method(iid2,lvmfit)
S3method(iid2,lvmfit2)
S3method(iidJack,default)
S3method(information,lvmfit2)
S3method(information2,lvmfit)
S3method(information2,lvmfit2)
S3method(leverage2,lvmfit)
S3method(leverage2,lvmfit2)
S3method(model.tables,lvmfit)
S3method(model.tables,lvmfit2)
S3method(model.tables2,lvmfit)
S3method(model.tables2,lvmfit2)
S3method(modelsearch2,lvmfit)
S3method(moments2,lvm)
S3method(moments2,lvmfit)
S3method(nStep,modelsearch2)
S3method(nobs2,lvmfit)
S3method(nobs2,lvmfit2)
S3method(print,intDensTri)
S3method(print,modelsearch2)
S3method(print,summary.glht2)
S3method(residuals,lvmfit2)
S3method(residuals2,lvmfit)
S3method(residuals2,lvmfit2)
S3method(sCorrect,default)
S3method(score,lvmfit2)
S3method(score2,lvmfit)
S3method(score2,lvmfit2)
S3method(setLink,lvm)
S3method(summary,calibrateType1)
S3method(summary,glht2)
S3method(summary,lvmfit2)
S3method(summary,modelsearch2)
S3method(summary2,lvmfit)
S3method(summary2,lvmfit2)
S3method(var2dummy,list)
S3method(var2dummy,lvm)
S3method(vcov,lvmfit2)
S3method(vcov2,lvmfit)
S3method(vcov2,lvmfit2)
export("sCorrect<-")
export(addLink)
export(calcDistMaxBootstrap)
export(calcDistMaxIntegral)
export(calcType1postSelection)
export(calibrateType1)
export(checkData)
export(clean)
export(coef2)
export(coefCov)
export(coefExtra)
export(coefIndexModel)
export(coefIntercept)
export(coefRef)
export(coefReg)
export(coefType)
export(coefVar)
export(combineFormula)
export(compare2)
export(confint2)
export(createContrast)
export(effects2)
export(estimate2)
export(extractData)
export(findNewLink)
export(formula2character)
export(gaussian_weight.estimate.hook)
export(gaussian_weight_gradient.lvm)
export(gaussian_weight_hessian.lvm)
export(gaussian_weight_logLik.lvm)
export(gaussian_weight_method.lvm)
export(gaussian_weight_objective.lvm)
export(gaussian_weight_score.lvm)
export(getNewLink)
export(getNewModel)
export(getStep)
export(getVarCov2)
export(glht2)
export(hessian2)
export(iid2)
export(iid2plot)
export(iidJack)
export(information2)
export(initVarLink)
export(initVarLinks)
export(intDensTri)
export(leverage2)
export(matrixPower)
export(model.tables2)
export(modelsearch2)
export(moments2)
export(nStep)
export(nobs2)
export(residuals2)
export(sCorrect)
export(sampleRepeated)
export(score2)
export(setLink)
export(skeleton)
export(summary2)
export(transformSummaryTable)
export(tryWithWarnings)
export(vcov2)
import(Rcpp)
import(lava)
importFrom(MASS,mvrnorm)
importFrom(Matrix,bdiag)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,autoplot)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,text)
importFrom(methods,is)
importFrom(multcomp,glht)
importFrom(mvtnorm,pmvnorm)
importFrom(mvtnorm,pmvt)
importFrom(mvtnorm,qmvnorm)
importFrom(mvtnorm,qmvt)
importFrom(mvtnorm,rmvnorm)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(reshape2,melt)
importFrom(sandwich,estfun)
importFrom(stats,anova)
importFrom(stats,coef)
importFrom(stats,confint)
importFrom(stats,cov)
importFrom(stats,effects)
importFrom(stats,formula)
importFrom(stats,logLik)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.tables)
importFrom(stats,predict)
importFrom(stats,qqnorm)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(stats,update)
importFrom(stats,vcov)
importFrom(utils,methods)
importFrom(utils,packageVersion)
importFrom(utils,setTxtProgressBar)
importFrom(utils,tail)
importFrom(utils,txtProgressBar)
useDynLib(lavaSearch2, .registration=TRUE)
