# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

OLS_cpp <- function(X, y) {
    .Call(`_lavaSearch2_OLS_cpp`, X, y)
}

OLS2_cpp <- function(X, y) {
    .Call(`_lavaSearch2_OLS2_cpp`, X, y)
}

wildBoot_cpp <- function(iid, lsIndexModel, nSample, nObs, nModel, p) {
    .Call(`_lavaSearch2_wildBoot_cpp`, iid, lsIndexModel, nSample, nObs, nModel, p)
}

