% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IntDensTri.R
\name{autoplot.intDensTri}
\alias{autoplot.intDensTri}
\title{2D-display of the Domain Used to Compute the Integral}
\usage{
\method{autoplot}{intDensTri}(object, coord.plot = c("x", "y1"), plot = TRUE, ...)
}
\arguments{
\item{object}{output of the function \code{intDensTri}.}

\item{coord.plot}{[character vector] the x and y coordinates. Can be \code{"x"}, \code{"y1"} to \code{"yd"}, \code{"z"} if \code{zmin} was specified when calling \code{intDensTri}.}

\item{plot}{[logical] should the plot be displayed?}

\item{...}{[internal] Only used by the generic method.}
}
\value{
A \code{ggplot} object.
}
\description{
2D-display of the domain used to compute the integral.
}
\seealso{
\code{\link{intDensTri}}
}
